import { lazy, memo, useEffect, useRef, useState } from "react";
import { SingleValue } from "react-select";
// import AsyncSelect from 'react-select/async';
import { Loader } from "@googlemaps/js-api-loader";
import type {ForwardRef as Step2ConfirmationForwardRef} from './Step2Confirmation';
const Step2Confirmation = lazy(()=>import('./Step2Confirmation'));

interface Address {
	place_id: string,
	formatted_address: string,
	lat: number,
	lng: number
}

export default memo(function Step2(){
	const step2ConfirmationRef = useRef<Step2ConfirmationForwardRef|null>(null);
	const mapRef = useRef<HTMLDivElement>(null);
	const [addressType, setAddressType] = useState<'home'|'hotel'>('home');
	const [differentDropOff, setDifferentDropOff] = useState(false);

	const [pickup, setPickup] = useState<Address>();
	const map = useRef<google.maps.Map|null>(null);
	const marker = useRef<google.maps.Marker|null>(null);

	const loader = new Loader({
		apiKey: process.env.NEXT_PUBLIC_GOOGLE_API_KEY as string,
		version: "weekly"
	});

	async function pickupOptions(inputValue: string){
		const {AutocompleteService} = await loader.importLibrary('places');
		const autocomplete = new AutocompleteService();
		const {predictions} = await autocomplete.getPlacePredictions({
			input: inputValue,
			componentRestrictions: { country: 'us' },
			locationRestriction: {
				north: 49.107278678461164,
				east: -117.0433480206906,
				south: 45.519004922782216,
				west: -125.03590159667671
			}
		});
		return predictions.map(v=>({ value: v.place_id, label: v.description }));
	}

	function onPickupChange(ev: SingleValue<{value: string, label: string}>){
		(async ()=>{
			const {Geocoder} = await loader.importLibrary('geocoding');
			const {geocode} = new Geocoder();
			const {results} = await geocode({ placeId: ev!.value });
			const result = results[0];
			const address = {
				place_id: result.place_id,
				formatted_address: result.formatted_address,
				lat: result.geometry.location.lat(),
				lng: result.geometry.location.lng()
			}
			setPickup(address);
			const latLng = { lat: address.lat, lng: address.lng }
			if(map.current){
				map.current.panTo(latLng);
				map.current.setZoom(16);
			}
			if(marker.current) marker.current.setMap(null);
			const {Marker, Animation} = await loader.importLibrary('marker');
			marker.current = new Marker({
				map: map.current,
				animation: Animation.DROP,
				position: latLng
			});
		})();
	}

	function next(){
		step2ConfirmationRef.current?.showModal();
	}

	useEffect(()=>{
		const latlng: google.maps.LatLngLiteral = { lat: 47.5, lng: -122 };
		const mapOptions: google.maps.MapOptions = {
			center:  latlng,
			zoom: 9,
			mapTypeId: 'roadmap',
			mapTypeControl: false,
			zoomControl: false,
			streetViewControl: false,
			fullscreenControl: false,
		};
		(async ()=>{
			const {Map} = await loader.importLibrary('maps');
			map.current = new Map(mapRef.current as HTMLDivElement, mapOptions);
		})()
		// eslint-disable-next-line react-hooks/exhaustive-deps
	}, []);

	return <>
		<section className="max-w-5xl mx-auto mt-12">
			<h1 className="text-2xl text-center font-bold leading-loose mb-10 text-brand-black">{addressType==='home' ? 'Find your address' : 'Search for your hotel'}</h1>

			<label className={`cursor-pointer flex items-center gap-x-2 justify-end pe-6 ${addressType==='home' ? 'visible' : 'invisible'}`}>
				<span className="label-text">Different drop-off</span> 
				<input type="checkbox" className="toggle bg-black checked:bg-brand-orange" defaultChecked={differentDropOff} onChange={(ev)=>{ setDifferentDropOff(ev.target.checked) }} />
			</label>

			<div className={`input input-bordered rounded-[0.3125rem] grid grid-cols-[1fr_auto] gap-x-2 justify-around items-center h-[3.75rem] mt-3`}>
				<label className={`grid justify-around items-center gap-x-2 ${ differentDropOff && addressType==='home' ? 'grid-cols-[auto_1fr_auto_1fr]' : 'grid-cols-[auto_1fr]'}`}>
					<img src="https://ik.imagekit.io/choreless/v2/icons/FigmaArrow2.svg" alt="icon" loading="lazy" />
					<div>
						<p className="text-xs text-[#808080]">Pickup</p>
						<input type="text" className="w-full h-full border-0 font-bold placeholder:font-normal focus:outline-none" placeholder="Enter Address" />
						{/* <AsyncSelect
							cacheOptions
							defaultOptions
							loadOptions={pickupOptions}
							onChange={onPickupChange}
							placeholder='Pickup'
							styles={{
								indicatorsContainer: (baseStyles) => ({
									...baseStyles,
									display: 'none'
								})
							}}
						/> */}
					</div>
					{
						differentDropOff && addressType==='home' &&
						<>
						<img src="https://ik.imagekit.io/choreless/v2/icons/FigmaArrow2.svg" alt="icon" loading="lazy" className="rotate-180" />
						<div>
							<p className="text-xs text-[#808080]">Drop-off</p>
							<input type="text" className="w-full h-full border-0 font-bold placeholder:font-normal focus:outline-none" placeholder="Enter Address" />
						</div>
						</>
					}
				</label>
				<button onClick={next} className="btn w-full rounded-[0.3125rem] px-6 text-white bg-brand-orange border-brand-orange hover:text-brand-orange hover:bg-transparent hover:border-brand-orange">Continue</button>
			</div>

			<div className="mx-2.5 mt-9 px-6 py-3.5 rounded-md flex justify-between text-brand-black bg-[#F3F3F6]" onClick={()=>{setAddressType(addressType==='home' ? 'hotel' : 'home')}}>
				<div>
					<h1 className="text-2xl font-bold leading-loose">{addressType==='home' ? 'Staying at a hotel' : 'Not in a hotel' } ?</h1>
					<p>{addressType==='home' ? 'No need to wait for your driver to arrive - leave the laundry at the reception' : 'If you are not a traveler, or not staying in a hotel, search by postcode or address.' }</p>
				</div>
				<img src={`https://ik.imagekit.io/choreless/v2/images/signup/${addressType==='home' ? 'FigmaWoman' : 'FigmaLocation'}.svg`} alt={addressType==='home' ? 'woman' : 'location'} loading="lazy" />
			</div>
		</section>
		<div className="fixed bottom-0 w-full h-[41vh]">
			<div ref={mapRef} className="h-full"></div>
		</div>
		<Step2Confirmation ref={step2ConfirmationRef} addressType={addressType} differentDropOff={differentDropOff} />
	</>
})
